#!/usr/local/bin/ruby -w
#
#	(c) 2014, Dirk Meyer, Im Grund 4, 34317 Habichtswald
#
def usage()
  STDERR.puts ""
  STDERR.puts "usage: #{$0} indexfile portdir >filename.dot"
  STDERR.puts ""
  STDERR.puts "	generate a graph of all depedencies"
  STDERR.puts ""
  exit 64
end

# Sample entry of INDEX file
#de-dict-1.2|/usr/ports/german/dict|/usr/local|Simple English/German dictionary|/usr/ports/german/dict/pkg-descr|ports@FreeBSD.org|german||agrep-2.04_2|http://www.de.freebsd.org/~wosch/dict/dict.cgi|||
INDEX_PKGNAME = 0
INDEX_PORTDIR = 1
INDEX_DEPENDS = 8

def read_index( indexfile )
  if not File.exists?( indexfile )
    STDERR.puts "INDEX file '#{indexfile}' not found"
    exit 66
  end

  $index_names = Hash.new
  $index_dirs = Hash.new
  $index_origin = Hash.new
  lines = File.read( indexfile ).split( "\n" )
  lines.each do |line|
    words = line.split( '|', -1 )
    # p words
    pkgname = words[ INDEX_PKGNAME ]
    portdir = words[ INDEX_PORTDIR ]
    depends = words[ INDEX_DEPENDS ]
    $index_names[ pkgname ] = depends
    $index_dirs[ portdir ] = pkgname
    $index_origin[ pkgname ] = portdir
  end
end

def clean_name( label )
  name = label.gsub( /[\/.-]/, '__' )
  return name
end

def make_node( pkgname, portdir )
  $seen[ pkgname ] = true
  origin = portdir.sub( "/usr/ports/", '' )
  name = clean_name( origin )
  buffer = " #{name} [label=\"#{origin}\"]\n"
  return buffer
end

def make_depends( pkgname, portdir )
  origin = portdir.sub( "/usr/ports/", '' )
  name = clean_name( origin )
  # p [ pkgname, $index_names[ pkgname ] ]
  depends = $index_names[ pkgname ].split( ' ' )
  depends.each do |depend|
    $depends.push( [ name, $index_origin[ depend ] ] )
  end
end

def add_node( pkgname, portdir )
  if $seen.has_key?( pkgname )
    return true
  end
  $seen[ pkgname ] = true
  $nodes.push( [ pkgname, portdir ] )
  return false
end

def search_node( pkgname, portdir )
  if add_node( pkgname, portdir )
    return ''
  end
  todo = Array.new
  depends = $index_names[ pkgname ].split( ' ' )
  depends.each do |depend|
    if add_node( depend, $index_origin[ depend ] )
      next
    end
    todo.push( depend )
  end
  todo.each do |depend|
    search_node( depend, $index_origin[ depend ] )
  end
end

def make_dot( pkgname, portdir )
  origin = portdir.sub( "/usr/ports/", '' )
  name = clean_name( origin )
  buffer = "digraph #{name}"
  buffer << ' {
graph [ bgcolor="#FFFFFF", ranksep="0.1", overlap="scalexy", splines="curved", size="100,100", sep="+1" ];
node [ shape="circle", fontcolor="#000000", color="#000000", fillcolor="#DDDDDD" ];
edge [ fontcolor="#00FF00", weight=100, w=1 ];
'

  $nodes.each do |node|
    pkgname, portdir = node
    buffer << make_node( pkgname, portdir )
    make_depends( pkgname, portdir )
  end

  $depends.each do |depend|
    parent, portdir = depend
    origin = portdir.sub( "/usr/ports/", '' )
    name = clean_name( origin )
    buffer << " #{parent} -> #{name}\n"
  end

  buffer << "\n}\n"
  return buffer
end

if ARGV[0].nil? or ARGV[1].nil?
  usage
end

indexfile = ARGV[0]
read_index( indexfile )
first_pkgname = nil
first_portdir = nil
$depends = Array.new
$nodes = Array.new
$seen = Hash.new

ARGV.shift
ARGV.each do |portdir|
  if not $index_dirs.has_key?( portdir )
    STDERR.puts "Index file '#{portdir}' not found in INDEX"
    exit 78
  end

  pkgname = $index_dirs[ portdir ]
  if first_pkgname.nil?
    first_pkgname = pkgname
  end
  if first_portdir.nil?
    first_portdir = portdir
  end
  search_node( pkgname, portdir )
end

puts make_dot( first_pkgname, first_portdir )

# eof
